'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.GlobalActionHandler = GlobalActionHandler;

var _lodash = require('lodash');

var _ = _interopRequireWildcard(_lodash);

var _reduxSaga = require('redux-saga');

var _effects = require('redux-saga/effects');

var _actionTypes = require('../constants/actionTypes');

var _libraryConstants = require('../constants/libraryConstants');

var _variables = require('../variables');

var _utils = require('../utils');

var _selectors = require('../selectors');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function* GlobalActionHandler() {
	yield* (0, _reduxSaga.takeEvery)(_actionTypes.GLOBAL_ACTION, handleGlobalAction);
}

/**
 * If we are the target then dispatch global action here, otherwise route it to its destination
 *
 * @param action Global action to handle
 */
function* handleGlobalAction(action) {
	const currentViewName = yield (0, _effects.select)(_selectors.getViewName);
	const target = action.payload.target;


	if ((0, _variables.areDebugStatementsEnabled)()) {
		console.log(`[GLOBAL ACTION]\ntype: ${ action.payload.action.type }\nwindow: ${ currentViewName }\ntarget: ${ target }`);
		(0, _utils.logToFile)(`[GLOBAL ACTION]\ntype: ${ action.payload.action.type }\nwindow: ${ currentViewName }\ntarget: ${ target }`);
	}

	// No matter what, if we are the target, dispatch here
	if (target === currentViewName) {
		yield dispatchGlobalActionLocally(currentViewName, action);
	}
	// Using `ELECTRON_PROCESS_NAME` as target will work even though the main process
	// may have been given a different name
	else if (target === _libraryConstants.ELECTRON_PROCESS_NAME && (0, _utils.isBrowserProcess)()) {
			yield dispatchGlobalActionLocally(currentViewName, action);
		}
		// Route action elsewhere (browser, or another view)
		else {
				// Add source to dispatched global actions
				if (action.payload.source == null || action.payload.source === '') {
					action.payload.source = currentViewName;
				}

				const browserName = (0, _utils.getGlobalVar)(_libraryConstants.GLOBAL_BROWSER_NAME_KEY);
				// If we are not the target, send action to the proper place

				if (target === browserName || target === _libraryConstants.ELECTRON_PROCESS_NAME) {
					yield sendGlobalActionToBrowser(currentViewName, action);
				} else {
					yield sendGlobalActionToRenderer(action);
				}
			}
}

/**
 * Dispatches global action payload to the local redux store
 *
 * @param currentViewName Current view name
 * @param action Action to dispatch
 */
function* dispatchGlobalActionLocally(currentViewName, action) {
	if ((0, _variables.areDebugStatementsEnabled)()) {
		console.log(`[GLOBAL ACTION]: Dispatching ${ action.payload.action.type } locally.`);
		(0, _utils.logToFile)(`[GLOBAL ACTION]: Dispatching ${ action.payload.action.type } locally.`);
	}

	// Check if we aren't dispatching global action to ourselves
	if (action.payload.source === currentViewName) {
		yield console.warn(`Global Action is being sent to its own view. Action: "${ action.type }", View: "${ currentViewName }"`);
	}

	// Dispatch action
	yield (0, _utils.sagaPut)(action.payload.action);
}

/**
 * Sends global action to the browser
 *
 * @param currentViewName Current view name
 * @param action Action to dispatch
 */
function* sendGlobalActionToBrowser(currentViewName, action) {
	if ((0, _variables.areDebugStatementsEnabled)()) {
		console.log(`[GLOBAL ACTION]: Dispatching ${ action.payload.action.type } to browser.`);
		(0, _utils.logToFile)(`[GLOBAL ACTION]: Dispatching ${ action.payload.action.type } to browser.`);
	}

	// If we are not sending the action from here
	if (action.payload.source !== currentViewName) {
		// Case when we are not the target and not the browser shouldn't happen
		yield console.warn(`Global Action ended up in a wrong place. Current: "${ currentViewName }", Source: "${ action.payload.source }", Target: "${ action.payload.target }"`);
	}

	// Send the action
	let ipc = yield (0, _utils.getIpc)();
	yield ipc.send(_libraryConstants.IPC_CHANNEL_NAME, action);
}

/**
 * Sends global action from browser to renderer
 *
 * @param action Global action to dispatch
 */
function* sendGlobalActionToRenderer(action) {
	const target = action.payload.target;


	if ((0, _variables.areDebugStatementsEnabled)()) {
		console.log(`[GLOBAL ACTION]: Dispatching ${ action.payload.action.type } to renderer "${ target }".`);
		(0, _utils.logToFile)(`[GLOBAL ACTION]: Dispatching ${ action.payload.action.type } to renderer "${ target }".`);
	}

	if ((0, _utils.isRendererWindowAvailable)(target)) {
		const window = yield (0, _utils.getWindowFromName)(target);
		const webcontents = window.webContents;
		yield webcontents.send(_libraryConstants.IPC_CHANNEL_NAME, action);
	} else {
		if ((0, _variables.areDebugStatementsEnabled)()) {
			console.log(`[GLOBAL ACTION]: Renderer "${ target }" not available.`);
			(0, _utils.logToFile)(`[GLOBAL ACTION]: Renderer "${ target }" not available.`);
		}

		// QQQ: Queue action?

		if ((0, _utils.getWindowIdByName)(target) > 0) {
			// We know this window but it doesn't exist anymore (no announce close dispatched)
			// TODO: Clear window
			// TODO: Dispatch announce close to others
		}
	}
}