'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.default = browserStoreEnhancer;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _reduceReducers = require('reduce-reducers');

var _reduceReducers2 = _interopRequireDefault(_reduceReducers);

var _libraryConstants = require('./constants/libraryConstants');

var _actions = require('./actions');

var _utils = require('./utils');

var _reducers = require('./reducers');

var _variables = require('./variables');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function browserStoreEnhancer() {
	let options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

	return storeCreator => {
		return function (reducer) {
			let initialState = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

			options = _immutable2.default.fromJS(options);

			options = checkOptions(options);

			(0, _variables.setReducerName)(options.get('reducerName'));
			if (options.getIn(['debug', 'enabled']) && options.getIn(['debug', 'debugStatementsEnabled'])) {
				(0, _variables.enableDebugStatements)();
			}

			// Check if our reducer is present
			if (!isOurReducerPresent(reducer)) {
				console.warn(`Can't find '${ (0, _variables.getReducerName)() }' in state. Make sure you've added electron-redux-multi-window-comm's reducer to your root reducer.`);
			}

			// Add options to initial state if necessary
			const processedSubscriptions = processSubscriptions(options);

			const initialStateWithOptions = addOptionsAndSubscriptionsToState(initialState, options, processedSubscriptions);

			// https://github.com/rackt/redux/issues/749#issuecomment-164327121
			const customReducer = (0, _reduceReducers2.default)(reducer, _reducers.stateChangeReducer);

			// Create final store
			let store = storeCreator(customReducer, initialStateWithOptions);

			let originDispatch = store.dispatch;

			// Setup ipc channel listener
			(0, _utils.setupIpcListener)(originDispatch);

			// Overwrite dispatch to handle diffing
			store.dispatch = action => {
				let previousState = store.getState();
				originDispatch(action);
				let nextState = store.getState();

				originDispatch((0, _actions.stateChanged)(previousState, nextState));
			};

			// Keep local reference to the store
			(0, _variables.setStore)(store);

			return store;
		};
	};
}

function isOurReducerPresent(reducer) {
	const type = '@@MultiElectron/PROBE_FOR_REDUCER_ACTION_' + Math.random().toString(36).substring(7).split('').join('.');

	const state = reducer({}, { type });

	return _lodash2.default.has(state, (0, _variables.getReducerName)());
}

function addOptionsAndSubscriptionsToState() {
	let initialState = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
	let options = arguments[1];
	let subscriptions = arguments.length <= 2 || arguments[2] === undefined ? _immutable2.default.Map() : arguments[2];

	if (_lodash2.default.has(initialState, (0, _variables.getReducerName)())) {
		// If initial state contains ElectronReduxComms's state we believe you know what you are doing, so we won't add
		// current options to it.
		return initialState;
	} else {
		let reducerState = _immutable2.default.fromJS({ options: options }).merge(subscriptions);
		return _lodash2.default.set(initialState, [(0, _variables.getReducerName)()], reducerState);
	}
}

function checkOptions(options) {
	// Check window name
	if (!_lodash2.default.isString(options.get('windowName')) || _lodash2.default.isEmpty(options.get('windowName'))) {
		// In case of missing `windowName` option in browser default to `ELECTRON_PROCESS_NAME`
		if ((0, _utils.isBrowserProcess)()) {
			console.warn(`The main Electron process wasn't given any name, '${ _libraryConstants.ELECTRON_PROCESS_NAME }' will be used instead.`);
			options = options.set('windowName', _libraryConstants.ELECTRON_PROCESS_NAME);
		} else {
			// QQQ: Generate random name?
			options = options.set('windowName', '');
			console.warn(`This view wasn't given any name. You won't be able to reference it from other views.`);
		}
	}

	// Check reducer name
	if (!_lodash2.default.isString(options.get('reducerName')) || _lodash2.default.isEmpty(options.get('reducerName'))) {
		options = options.set('reducerName', _libraryConstants.REDUCER_NAME);
	}

	// Check subscribe to
	if (!_immutable2.default.List.isList(options.get('subscribeTo'))) {
		options = options.set('subscribeTo', _immutable2.default.List());
	}

	return options;
}

function processSubscriptions(options) {
	const subscribeTo = options.get('subscribeTo', _immutable2.default.List());

	let state = _immutable2.default.Map();

	for (var i = 0; i < subscribeTo.size; i++) {
		const subscribeInfo = subscribeTo.get(i, _immutable2.default.Map());
		const targetWindow = subscribeInfo.get('windowName');

		if (subscribeInfo.has('stateFilter')) {
			state = (0, _reducers.subscriptionsReducer)(state, (0, _actions.subscribeToViewState)(targetWindow, subscribeInfo.get('stateFilter').toJS()));
		}

		if (subscribeInfo.has('actionTypes')) {
			state = (0, _reducers.subscriptionsReducer)(state, (0, _actions.subscribeToViewActions)(targetWindow, subscribeInfo.get('actionTypes').toJS()));
		}
	}

	return state;
}