'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.processShapeEditPath = processShapeEditPath;

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Path separator
const SEPARATOR = '/';

/**
 * Transforms `path` based on the given `edit` into a gettable array path
 * 
 * Example:
 * '/x' + ['a', 'b', 'c'] => ['x', 'b', 'c']
 * 
 * @param edit
 * @param path
 * @returns {*}
 */
function processShapeEditPath() {
	let edit = arguments.length <= 0 || arguments[0] === undefined ? '' : arguments[0];
	let path = arguments.length <= 1 || arguments[1] === undefined ? _immutable2.default.List() : arguments[1];

	// If is a ordinary array convert it to Immutable
	if ((0, _isArray2.default)(path)) {
		path = _immutable2.default.fromJS(path);
	}

	// Return unchanged path on incorrect parameters
	if (edit.length <= 0 || path.size <= 0) {
		return path;
	}

	// Handle escaped "/"
	// Check for "\/" and replace it with "\\\"
	var segmentIndex = 0;
	var escapeLocations = [];
	for (var i = 0; i < edit.length; i++) {
		if (edit[i] === '/') {
			segmentIndex++;
		} else if (edit[i] === '\\' && edit[i + 1] === '/') {
			escapeLocations.push(segmentIndex);
			edit = edit.substr(0, i) + '\\\\\\' + edit.substr(i + 2);

			// Fix current index since we are editing the string we are iterating over
			// -1 because we are cutting out the /
			i = i + '\\\\\\'.length - 1;
		}
	}

	segmentIndex = null;

	let editSegments = edit.split(SEPARATOR);

	// Replace back escaped "/"
	// "\\\" => "/"
	if (escapeLocations.length > 0) {
		escapeLocations.forEach(location => {
			editSegments[location] = editSegments[location].replace('\\\\\\', '/');
		});
	}

	// Edit is an exact path prefixed with "="
	if (editSegments[0] === '=') {
		return editSegments.length > 1 ? _immutable2.default.fromJS(editSegments.slice(1)) : _immutable2.default.List();
	}

	// We need to know if we are starting from root or the end to correctly place the initial index
	// Also "+" depends on this, because if true we add before current index and if false we add after
	let fromRoot = edit[0] === SEPARATOR;
	let currentPathIndex = fromRoot ? 0 : path.size - 1;

	let editedPath = path;

	/**
  * The reason we are not checking if `editSegmentsIndex` is in `editPath`' range of indexes is that
  * we need to go beyond this range in order for these to work:
  * 
  * '-/-/-' + ['a', 'b', 'c'] => []
  * '/?/?/?/+x' + ['a', 'b', 'c'] => ['a', 'b', 'c', 'x']
  * 
  * When we actually need the index to be in the proper range we force it there with `forceInRange`
  */
	for (var editSegmentsIndex = 0; editSegmentsIndex < editSegments.length; editSegmentsIndex++) {
		let editSegment = editSegments[editSegmentsIndex];

		// Handle empty segments //
		if (editSegment.length <= 0) {
			// When using / at the start the split will always add empty segment as the first segment
			// /a => ['', 'a']
			if (editSegmentsIndex !== 0) {
				// 				editSegmentsIndex++;

				// Treat // same as ?
				currentPathIndex++;
			}

			continue;
		}

		switch (editSegment[0]) {
			// Add new path segment
			case '+':
				{
					let segmentName = editSegment.slice(1);

					// fromRoot = true => we add before current segment, if false we add after
					let insertPosition = fromRoot ? currentPathIndex : currentPathIndex + 1;

					editedPath = insertIntoList(segmentName, insertPosition, editedPath);

					currentPathIndex = forceInRange(currentPathIndex, 0, editedPath.size - 1);
					currentPathIndex++;
					break;
				}

			// Remove current segment
			case '-':
				{
					currentPathIndex = forceInRange(currentPathIndex, 0, editedPath.size - 1);
					editedPath = editedPath.delete(currentPathIndex);
					currentPathIndex = forceInRange(currentPathIndex, 0, editedPath.size - 1);
					break;
				}

			// Go the the end (and set fromRoot as false)
			case '*':
				{
					currentPathIndex = editedPath.size - 1;
					fromRoot = false;
					break;
				}

			// Go to the next segment (opposite of ..)
			case '?':
				{
					currentPathIndex++;
					break;
				}

			// Check for ".."
			case '.':
				{
					// Go up by one (opposite of ?)
					if (editSegment.length === 2 && editSegment[0] === '.' && editSegment[1] === '.') {
						currentPathIndex--;
						continue;
					}

					// Continue below to rename otherwise
					// 				break;
				}

			// Escaped segment name that starts with any of the operation symbols (|+path => +path)
			// De-escape it and pass it below to rename
			case '|':
				{
					if (editSegment[0] === '|') {
						editSegment = editSegment.slice(1);
					}
				}

			// Set name of the current path segment
			default:
				{
					currentPathIndex = forceInRange(currentPathIndex, 0, editedPath.size - 1);
					editedPath = editedPath.set(currentPathIndex, editSegment);
					currentPathIndex++;
				}
		}
	}

	return editedPath;
}

/**
 * Sets `value` back to the given `min` - `max` range if necessary
 * 
 * @param value
 * @param min
 * @param max
 * @returns {*}
 */
function forceInRange(value, min, max) {
	if (value > max) {
		return max;
	} else if (value < min) {
		return min;
	} else {
		return value;
	}
}

/**
 * Insert `item` into the `list` at a given `position`
 * 
 * @param item
 * @param position
 * @param list
 * @returns {Array.<T>|string|Buffer}
 */
function insertIntoList(item, position, list) {
	var before = list.slice(0, position);
	var after = list.slice(position);

	return before.push(item).concat(after);
}