'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.processShapeMapPath = processShapeMapPath;

var _listInfo = require('./listInfo');

var ListInfo = _interopRequireWildcard(_listInfo);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

// TODO: Figure out error return values (empty array?)
function processShapeMapPath(path, state) {
	let selectRoot = arguments.length <= 2 || arguments[2] === undefined ? _immutable2.default.List() : arguments[2];

	let processedPath = _immutable2.default.List();

	if (path == null || path.size <= 0 || state == null) {
		return _immutable2.default.List();
	}

	/**
  * Tracks if the path contained at least one list that isn't slice list
  *
  * OK:
  * ['a', 'b']
  * ['a', 'b', [0,1]]
  *
  * NOT OKAY:
  * ['a', 'b', ['c']]
  *
  * It's used for performance improvements. We don't actually copy `pathItem`s
  * into `processedPath` until we encounter the first nested list. At that time
  * we slice the preceding part of `path` onto the `processedPath`.
  *
  * @type {boolean}
  */
	let wasNestedListInPath = false;

	/**
  * Tracks if path contains no lists of any type.
  *
  * OK:
  * ['a', 'b']
  *
  * It's used to determine whether we need to `get` the whole path
  *
  * @type {boolean}
  */
	let isPurePath = true;

	/**
  * Goes through the path list and recursively calls itself if it encounters
  * a nested list (not slice).
  */
	for (var pathIndex = 0; pathIndex < path.size; pathIndex++) {
		let pathItem = path.get(pathIndex);

		if (_immutable2.default.List.isList(pathItem)) {
			// We have a nested list here
			if (!ListInfo.isSliceList(pathItem)) {
				let processedItem = processShapeMapPath(pathItem, state, selectRoot);

				if (!wasNestedListInPath) {
					processedPath = path.slice(0, pathIndex);

					wasNestedListInPath = true;
				}

				pathItem = processedItem;
			}

			// Not a pure path now, since we just had a list
			isPurePath = false;
		}

		// Don't put any pathItems into processedPath until we've encountered a nested list
		if (wasNestedListInPath) {
			processedPath = processedPath.push(pathItem);
		}
	}

	// Handle edge cases + prepend `selectRoot`

	// Path is a simple get
	if (isPurePath) {
		processedPath = processedPath.clear().push(state.getIn(selectRoot.concat(path)));
	}
	// In case when there are slice lists but no nested lists in the path we have
	// to populate `processedPath` with path since we don't copy anything until we
	// encounter a nested list.
	else if (!wasNestedListInPath) {
			processedPath = selectRoot.concat(path);
		} else {
			processedPath = selectRoot.concat(processedPath);
		}

	let outputPath = _immutable2.default.fromJS([[]]);

	/**
  * Map `processedPathItem`s to each other and process slice lists
  * At this point the `processedPath` can only contain:
  * - gettable string or number item
  * - list with gettable string or number items
  * - slice list
  *
  */
	for (var processedPathIndex = 0; processedPathIndex <= processedPath.size; processedPathIndex++) {
		let processedPathItem = processedPath.get(processedPathIndex);

		/**
   * We are handling two situations here:
   *
   * 1) Slice list
   * When we encounter a slice list we `get` all the entries in `outputPath` and run slice on them
   * Currently when the returned value isn't a list, we use it too, but that might change in future.
   *
   * 2) Final get
   * Unless the last item was a slice we need to get all the paths, since we always return the final
   * values and not selectors.
   *
   */
		if (ListInfo.isSliceList(processedPathItem) || processedPathIndex === processedPath.size) {
			let gotPaths;
			// Final get
			if (processedPathIndex === processedPath.size) {
				gotPaths = getAllPathsAndOptionallySlice(outputPath, state);
			}
			// Slice list
			else {
					gotPaths = getAllPathsAndOptionallySlice(outputPath, state, processedPathItem);
				}

			// Reset the current `outputPath` since it's obsolete now
			outputPath = _immutable2.default.fromJS([[]]);
			// Update the `processedPathItem` with the new paths
			processedPathItem = gotPaths;
		}

		let tempOutputPath = _immutable2.default.List();

		tempOutputPath = tempOutputPath.withMutations(mutable => {

			// Map `processedPathItem` value to the current contents of outputPath
			//
			// Direct value:
			// [['a'], ['b']] + 'c' =>
			// [['a', 'c'], ['b', 'c']]
			//
			// List:
			// [['a'], ['b']] + ['c', 'd'] =>
			// [['a', 'c'], ['a', 'd'], ['b', 'c'], ['b', 'd']]
			outputPath.forEach(a => {
				// List:
				if (_immutable2.default.List.isList(processedPathItem)) {
					processedPathItem.forEach(b => {
						mutable.push(a.concat(b));
					});
				}
				// Direct value:
				else {
						mutable.push(a.concat(processedPathItem));
					}
			});
		});

		outputPath = tempOutputPath;

		// We don't need to get the paths when we have a pure path or when the last item
		// was a slice
		if (processedPathIndex === processedPath.size - 1 && (isPurePath || ListInfo.isSliceList(processedPath.get(processedPathIndex)))) {
			break;
		}
	}

	return outputPath;
}

// Get all the paths and put their contents to a single list (and optionally slice
// the contents)
function getAllPathsAndOptionallySlice(paths, state) {
	let slice = arguments.length <= 2 || arguments[2] === undefined ? _immutable2.default.List() : arguments[2];

	let gotPaths = _immutable2.default.List();

	gotPaths = gotPaths.withMutations(mutable => {
		paths.forEach(gen => {
			if (state.hasIn(gen)) {
				let got = state.getIn(gen);
				if (_immutable2.default.List.isList(got)) {
					if (slice.size > 0) {
						mutable.push(...got.slice(...slice));
					} else {
						mutable.push(...got);
					}
				} else {
					mutable.push(got);
				}
			}
		});
	});

	return gotPaths;
}